/*
This is the LnkBox module for LiteStep.

Copyright (C) 2001 Mike Solomon

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#define WIN32_LEAN_AND_MEAN

#include <windows.h>
#include <shellapi.h>
#include "..\lsapi\lsapi.h"
#include "lnkbox.h"

/* our class name and revision ids */
const char szAppName[] = "LnkBox";

const char rcsRevision[] = "$Revision: 0.1$";
const char rcsId[] = "$Id: LnkBox 0.1 Allout$";

/* other constants */
const int LEAVE_TIMER_ID = 1;

/* object handles */
HDC hMemItemDC, hMemSelDC;
HFONT hFont;
HWND hWnd, hParentWnd;

/* user settings */
char szDir[MAX_PATH] = "c:\\windows\\desktop";

int cxItem = 120;
int cyItem = 20;
int iFontSize = 12;
int nDisplay = 10;
int xMain = 0;
int yMain = 0;
int iOffsetTop = 0;
int iOffsetLeft = 0;
int iOffsetBottom = 0;
int iOffsetRight = 0;

BOOL bOnTop = FALSE;
BOOL bStartHidden = FALSE;

COLORREF colorItemText = 0x000000;
COLORREF colorSelText = 0xFFFFFF;

/* internal variables */
char szLitestepPath[MAX_PATH];
char szLnks[64][MAX_PATH];

int cxMain = 0;
int cyMain = 0;

int iSel = -1;
int nItems = 0;

BOOL bLeaveTimer = FALSE;
BOOL bHidden = FALSE;
BOOL bWharf = FALSE;

/* function prototypes */
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
void ReadSettings(void);
void LnkSort(void);
void RegisterBangCommands(void);
void UnregisterBangCommands(void);
void BangRefresh(HWND caller, LPCSTR args);
void BangSetDir(HWND caller, LPCSTR args);
void BangShow(HWND caller, LPCSTR args);
void BangHide(HWND caller, LPCSTR args);
void BangToggle(HWND caller, LPCSTR args);

int initModule(HWND ParentWnd, HINSTANCE dllInst, wharfDataType* wd)
{
	return initModuleEx(ParentWnd, dllInst, wd->lsPath);
}

int initWharfModule(HWND ParentWnd, HINSTANCE dllInst, wharfDataType *wd)
{
	bWharf = TRUE;
	return initModuleEx (ParentWnd, dllInst, wd->lsPath);
}

int initModuleEx(HWND ParentWnd, HINSTANCE dllInst, LPCSTR szPath)
{
	WNDCLASS wc;
	UINT Msgs[10];
	HWND LitestepWnd;

	if(bWharf)
	{
		hParentWnd = ParentWnd;
	}
	else
	{	
		hParentWnd = FindWindow("DesktopBackgroundClass", NULL);
		if(!hParentWnd) hParentWnd = GetDesktopWindow();
	}

	wc.style = 0;
	wc.lpfnWndProc = WndProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = dllInst;
	wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH) GetStockObject(WHITE_BRUSH);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = szAppName;

	strcpy(szLitestepPath, szPath);

	ReadSettings();

	if(!RegisterClass(&wc))
	{
		MessageBox(hParentWnd, "Failure registering window class.", szAppName, MB_OK);

		PostQuitMessage(0);
		return 1;
	}

	if(bWharf)
	{
		hWnd = CreateWindowEx(
			WS_EX_TRANSPARENT,
			szAppName,
			szAppName,
			WS_CHILD,
			xMain,
			yMain,
			cxMain,
			cyMain,
			hParentWnd,
			NULL,
			dllInst,
			NULL
		);
	}
	else
	{
		hWnd = CreateWindowEx(
			WS_EX_TOOLWINDOW,
			szAppName,
			szAppName,
			WS_POPUP,
			xMain,
			yMain,
			cxMain,
			cyMain,
			NULL,
			NULL,
			dllInst,
			NULL
		);
	}

	if(!hWnd)
	{
		MessageBox(hParentWnd, "Failure creating window.", szAppName, MB_OK);

		PostQuitMessage(0);
		return 1;
	}

	Msgs[0] = LM_GETREVID;
	Msgs[1] = 0;

	LitestepWnd = GetLitestepWnd();

	SendMessage(LitestepWnd, LM_REGISTERMESSAGE, (WPARAM) hWnd, (LPARAM) Msgs);

	RegisterBangCommands();

	SetWindowLong(hWnd, GWL_USERDATA, magicDWord);

	if(bOnTop) SetWindowPos(hWnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);

	if(bWharf)
	{
		ShowWindow(hWnd, SW_SHOWNORMAL);
	}
	else
	{
		if(bStartHidden) bHidden = TRUE; else ShowWindow(hWnd, SW_SHOWNORMAL);
	}

	return 0;
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	HDC hDC;
	PAINTSTRUCT ps;
	POINT pt;
	RECT rcRect;
	int nCount;
	int iTemp;
	char szTemp[MAX_PATH];
	PSTR szBuffer;

	switch(message)
	{
	case WM_CLOSE:
		return 0;
	case WM_TIMER:
		if(wParam != LEAVE_TIMER_ID) return 0;

		GetCursorPos(&pt);
		GetClientRect(hWnd, &rcRect);

		ScreenToClient(hWnd, &pt);

		if(!PtInRect(&rcRect, pt))
		{
			iSel = -1;

			InvalidateRect(hWnd, NULL, FALSE);

			KillTimer(hWnd, LEAVE_TIMER_ID);
			bLeaveTimer = FALSE;
		}

		return 0;
	case WM_MOUSEMOVE:
		if(wParam) return 0;

		iTemp = HIWORD(lParam) / cyItem;
		if(iTemp != iSel)
		{
			if(iTemp < nItems)
			{
				iSel = iTemp;
				InvalidateRect(hWnd, NULL, FALSE);
				if(!bLeaveTimer)
				{
					SetTimer(hWnd, LEAVE_TIMER_ID, 20, NULL);
					bLeaveTimer = TRUE;
				}
			}
			else
			{
				iSel = -1;
				InvalidateRect(hWnd, NULL, FALSE);
				if(bLeaveTimer)
				{
					KillTimer(hWnd, LEAVE_TIMER_ID);
					bLeaveTimer = FALSE;
				}
			}
		}

		return 0;
	case WM_PAINT:
		hDC = BeginPaint(hWnd, &ps);

		SetBkMode(hDC, TRANSPARENT);

		SelectObject(hDC, hFont);

		for(nCount = 0; nCount < nItems; nCount++)
		{
			rcRect.top = nCount * cyItem + iOffsetTop;
			rcRect.left = iOffsetLeft;
			rcRect.bottom = (nCount + 1) * cyItem - iOffsetBottom;
			rcRect.right = cxItem - iOffsetRight;

			if(iSel == nCount)
			{
				SetTextColor(hDC, colorSelText);
				BitBlt(hDC, 0, nCount * cyItem, cxItem, cyItem, hMemSelDC, 0, 0, SRCCOPY);
			}
			else
			{
				SetTextColor(hDC, colorItemText);
				BitBlt(hDC, 0, nCount * cyItem, cxItem, cyItem, hMemItemDC, 0, 0, SRCCOPY);
			}

			DrawText(hDC, szLnks[nCount], -1, &rcRect, DT_SINGLELINE | DT_TOP | DT_LEFT | DT_END_ELLIPSIS);
		}

		for(nCount = nItems; nCount < nDisplay; nCount++)
		{
			BitBlt(hDC, 0, nCount * cyItem, cxItem, cyItem, hMemItemDC, 0, 0, SRCCOPY);
		}

		EndPaint(hWnd, &ps);

		return 0;
	case WM_LBUTTONDOWN:
		if(iSel != -1)
		{
			strcpy(szTemp, szDir);
			strcat(szTemp, szLnks[iSel]);
			strcat(szTemp, ".lnk");
			ShellExecute(hWnd, NULL, szTemp, NULL, NULL, SW_SHOWNORMAL);
		}

		return 0;
	case LM_GETREVID:
			szBuffer = (PSTR) lParam;

			if(wParam == 0)
			{
				strcpy(szBuffer, "LnkBox: ");
				strcat(szBuffer, &rcsRevision[11]);
				szBuffer[strlen(szBuffer) - 1] = 0;
			}
			else if (wParam == 1)
			{
				strcpy(szBuffer, &rcsId[5]);
				szBuffer[strlen(szBuffer) - 1] = 0;
			}
			else
			{
				strcpy(szBuffer, "");
			}

			return strlen(szBuffer);
	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
}

void ReadSettings(void)
{
	BOOL bNext;
	HANDLE hFindFile;
	HBITMAP hBitmap;
	HBRUSH hBrush;
	HDC hParentDC;
	HPEN hPen;
	WIN32_FIND_DATA fdFindData;
	COLORREF rcColor;
	RECT rcRect;
	char szTemp[MAX_PATH];
	int iFontHeight;
	
	hParentDC = GetDC(hParentWnd);

	cxItem = GetRCInt("LnkBoxWidth", 120);
	cyItem = GetRCInt("LnkBoxHeight", 20);
	iFontSize = GetRCInt("LnkBoxFontSize", 12);
	nDisplay = GetRCInt("LnkBoxItems", 10);
	
	GetClientRect(hParentWnd, &rcRect);

	xMain = GetRCInt("LnkBoxX", 0);
	yMain = GetRCInt("LnkBoxY", 0);

	if(xMain < 0) xMain += rcRect.right;
	if(yMain < 0) yMain += rcRect.bottom;
	
	iOffsetTop = GetRCInt("LnkBoxOffsetTop", 0);
	iOffsetLeft = GetRCInt("LnkBoxOffsetLeft", 0);
	iOffsetBottom = GetRCInt("LnkBoxOffsetBottom", 0);
	iOffsetRight = GetRCInt("LnkBoxOffsetRight", 0);

	bOnTop = GetRCBool("LnkBoxAlwaysOnTop", TRUE);
	bStartHidden = GetRCBool("LnkBoxStartHidden", TRUE);

	cxMain = cxItem;
	cyMain = cyItem * nDisplay;

	/* read lnks into array */
	GetRCString("LnkBoxDir", szDir, "c:\\windows\\desktop", MAX_PATH);

	if(szDir[strlen(szDir) - 1] != '\\') strcat(szDir, "\\");
	strcpy(szTemp, szDir);
	strcat(szTemp, "*.lnk");

	nItems = 0;

	hFindFile = FindFirstFile(szTemp, &fdFindData);
	if(hFindFile)
	{
		nItems = 1;
		strncpy(szLnks[0], fdFindData.cFileName, strlen(fdFindData.cFileName) - 4);

		bNext = TRUE;

		while(bNext)
		{
			bNext = FindNextFile(hFindFile, &fdFindData);
			if(bNext)
			{
				nItems++;
				strncpy(szLnks[nItems - 1], fdFindData.cFileName, strlen(fdFindData.cFileName) - 4);
			}
		}
	}

	FindClose(hFindFile);

	LnkSort();

	/* create memory image for item */
	GetRCString("LnkBoxItemBitmap", szTemp, ".none", MAX_PATH);
	rcColor = GetRCColor("LnkBoxItemColor", 0xFFFFFF);
	
	hMemItemDC = CreateCompatibleDC(hParentDC);
	hBitmap = LoadLSImage(szTemp, NULL);

	if(hBitmap)
	{
		SelectObject(hMemItemDC, hBitmap);
	}
	else
	{
		hBitmap = CreateCompatibleBitmap(hParentDC, cxItem, cyItem);
		hBrush = CreateSolidBrush(rcColor);
		hPen = CreatePen(PS_SOLID, 1, rcColor);
		SelectObject(hMemItemDC, hBitmap);
		SelectObject(hMemItemDC, hBrush);
		SelectObject(hMemItemDC, hPen);
		Rectangle(hMemItemDC, 0, 0, cxItem, cyItem);
	}

	DeleteObject(hBitmap);
	DeleteObject(hBrush);
	DeleteObject(hPen);

	/* create memory image for selected item */
	GetRCString("LnkBoxSelBitmap", szTemp, ".none", MAX_PATH);
	rcColor = GetRCColor("LnkBoxSelColor", 0x808080);

	hMemSelDC = CreateCompatibleDC(hParentDC);
	hBitmap = LoadLSImage(szTemp, NULL);

	if(hBitmap)
	{
		SelectObject(hMemSelDC, hBitmap);
	}
	else
	{
		hBitmap = CreateCompatibleBitmap(hParentDC, cxItem, cyItem);
		hBrush = CreateSolidBrush(rcColor);
		hPen = CreatePen(PS_SOLID, 1, rcColor);
		SelectObject(hMemSelDC, hBitmap);
		SelectObject(hMemSelDC, hBrush);
		SelectObject(hMemSelDC, hPen);
		Rectangle(hMemSelDC, 0, 0, cxItem, cyItem);
	}

	DeleteObject(hBitmap);
	DeleteObject(hBrush);
	DeleteObject(hPen);

	/* calculate pixel height from pt size */
	GetRCString("LnkBoxFont", szTemp, "Arial", MAX_PATH);
	iFontHeight = -((iFontSize * GetDeviceCaps(hParentDC, LOGPIXELSY)) / 72);

	/* create font */
	hFont = CreateFont(iFontHeight, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, szTemp);
	
	colorItemText = GetRCColor("LnkBoxItemTextColor", 0x000000);
	colorSelText = GetRCColor("LnkBoxSelTextColor", 0x000000);	

	ReleaseDC(hParentWnd, hParentDC);
}

void LnkSort(void)
{
	char szTemp[MAX_PATH];
	int nCount;

	for(nCount = 1; nCount < nItems; nCount++)
	{
		if(stricmp(szLnks[nCount], szLnks[nCount - 1]) < 0)
		{
			strcpy(szTemp, szLnks[nCount]);
			strcpy(szLnks[nCount], szLnks[nCount - 1]);
			strcpy(szLnks[nCount - 1], szTemp);
			nCount = 0;
		}
	}
}

void BangRefresh(HWND caller, LPCSTR args)
{
	BOOL bNext;
	HANDLE hFindFile;
	WIN32_FIND_DATA fdFindData;
	char szTemp[MAX_PATH];

	/* read lnks into array */
	if(szDir[strlen(szDir) - 1] != '\\') strcat(szDir, "\\");
	strcpy(szTemp, szDir);
	strcat(szTemp, "*.lnk");

	/* clear array */
	memset(szLnks, 0, sizeof(szLnks));
	
	nItems = 0;

	hFindFile = FindFirstFile(szTemp, &fdFindData);
	if(hFindFile)
	{
		nItems = 1;
		strncpy(szLnks[0], fdFindData.cFileName, strlen(fdFindData.cFileName) - 4);

		bNext = TRUE;

		while(bNext)
		{
			bNext = FindNextFile(hFindFile, &fdFindData);
			if(bNext)
			{
				nItems++;
				strncpy(szLnks[nItems - 1], fdFindData.cFileName, strlen(fdFindData.cFileName) - 4);
			}
		}
	}

	FindClose(hFindFile);

	LnkSort();

	InvalidateRect(hWnd, NULL, FALSE);
	UpdateWindow(hWnd);
}

void BangSetDir(HWND caller, LPCSTR args)
{
	strcpy(szDir, args);

	BangRefresh(hWnd, NULL);
}

void BangShow(HWND caller, LPCSTR args)
{
	bHidden = FALSE;
	ShowWindow(hWnd, SW_SHOWNORMAL);
}

void BangHide(HWND caller, LPCSTR args)
{
	bHidden = TRUE;
	ShowWindow(hWnd, SW_HIDE);
}

void BangToggle(HWND caller, LPCSTR args)
{
	if(bHidden)
	{
		bHidden = FALSE;
		ShowWindow(hWnd, SW_SHOWNORMAL);
	}
	else
	{
		bHidden = TRUE;
		ShowWindow(hWnd, SW_HIDE);
	}
}

void RegisterBangCommands(void)
{
	AddBangCommand("!LnkBoxRefresh", BangRefresh);
	AddBangCommand("!LnkBoxSetDir", BangSetDir);
	AddBangCommand("!LnkBoxShow", BangShow);
	AddBangCommand("!LnkBoxHide", BangHide);
	AddBangCommand("!LnkBoxToggle", BangToggle);
}

void UnregisterBangCommands(void)
{
	RemoveBangCommand("!LnkBoxRefresh");
	RemoveBangCommand("!LnkBoxSetDir");
	RemoveBangCommand("!LnkBoxShow");
	RemoveBangCommand("!LnkBoxHide");
	RemoveBangCommand("!LnkBoxToggle");
}

int quitWharfModule(HINSTANCE dllInst)
{
	return quitModule(dllInst);
}

int quitModule(HINSTANCE dllInst)
{
	UINT Msgs[10];
	HWND LitestepWnd;

	DeleteDC(hMemItemDC);
	DeleteDC(hMemSelDC);
	DeleteObject(hFont);
	KillTimer(hWnd, LEAVE_TIMER_ID);

	LitestepWnd = GetLitestepWnd();

	Msgs[0] = LM_GETREVID;
	Msgs[1] = 0;

	SendMessage(LitestepWnd, LM_UNREGISTERMESSAGE, (WPARAM) hWnd, (LPARAM) Msgs);

	DestroyWindow(hWnd);
	UnregisterBangCommands();
	UnregisterClass(szAppName, dllInst);

	return 0;
}


